@@ -46,7 +46,7 @@ class OsRelease:
     def _load_any(self):
         # type: () -> None
         for path in ('/etc/os-release', '/usr/lib/os-release'):
-            self._data = self._load('/etc/os-release')
+            self._data = self._load(path)
 
             if self._data:
                 return
@@ -184,6 +184,39 @@ def is_glvnd():
         return False
 
 
+def is_package_available(package_name):
+    try:
+        subprocess.run(['apt-cache', 'show', package_name],
+                       stdout=subprocess.DEVNULL, check=True)
+        return True
+    except (OSError, FileNotFoundError, subprocess.CalledProcessError):
+        return False
+
+
+def choose_xdg_portal_backend(default_backend="xdg-desktop-portal-gtk"):
+    gtk_desktop = ["Cinnamon", "GNOME", "LXDE", "MATE", "Unity", "XFCE"]
+    kde_desktop = ["KDE"]
+    gtk_backend = "xdg-desktop-portal-gtk"
+    kde_backend = "xdg-desktop-portal-kde"
+
+    current_desktops = os.environ.get("XDG_CURRENT_DESKTOP", "").split(":")
+
+    for c in current_desktops:
+        if c in gtk_desktop:
+            if is_package_available(gtk_backend):
+                return gtk_backend
+        elif c in kde_desktop:
+            if is_package_available(kde_backend):
+                return kde_backend
+
+    if is_package_available(default_backend):
+        return default_backend
+    else:
+        sys.stderr.write(
+            "There isn't a known XDG portal backend that can be installed")
+        return None
+
+
 def remap_package(name):
     if name in (
             'python-apt',
@@ -251,10 +284,23 @@ def create_package(description):
 
     name = remap_package(name)
 
+    if name in (
+            "xdg-desktop-portal-gtk",
+            "xdg-desktop-portal-kde",
+    ):
+        name = choose_xdg_portal_backend(name)
+        # Skip version conditions for portal backends because we don't know
+        # in advance which backend we will need to install.
+        # In the future, if necessary, we can enhance this check and, for
+        # example, embed a minimum version for each possible backed.
+        version_conditions = []
+
     if name is None:
         return None
     elif multiarch is not None:
         description = name + ':' + multiarch
+    else:
+        description = name
 
     return Package(description, version_conditions)
 
